/**
 * Regular expression that matches one or more `AttValue` characters in a
 * double-quoted attribute value.
 *
 * @see https://www.w3.org/TR/2008/REC-xml-20081126/#NT-AttValue
 */
export declare const attValueCharDoubleQuote: RegExp;
/**
 * Regular expression that matches one or more `AttValue` characters in a
 * single-quoted attribute value.
 *
 * @see https://www.w3.org/TR/2008/REC-xml-20081126/#NT-AttValue
 */
export declare const attValueCharSingleQuote: RegExp;
/**
 * Regular expression that matches a whitespace character that should be
 * normalized to a space character in an attribute value.
 *
 * @see https://www.w3.org/TR/2008/REC-xml-20081126/#AVNormalize
 */
export declare const attValueNormalizedWhitespace: RegExp;
/**
 * Regular expression that matches one or more characters that signal the end of
 * XML `CharData` content.
 *
 * @see https://www.w3.org/TR/2008/REC-xml-20081126/#dt-chardata
 */
export declare const endCharData: RegExp;
/**
 * Mapping of predefined entity names to their replacement values.
 *
 * @see https://www.w3.org/TR/2008/REC-xml-20081126/#sec-predefined-ent
 */
export declare const predefinedEntities: Readonly<{
    [name: string]: string;
}>;
/**
 * Returns `true` if _char_ is an XML `NameChar`, `false` if it isn't.
 *
 * @see https://www.w3.org/TR/2008/REC-xml-20081126/#NT-NameChar
 */
export declare function isNameChar(char: string): boolean;
/**
 * Returns `true` if _char_ is an XML `NameStartChar`, `false` if it isn't.
 *
 * @see https://www.w3.org/TR/2008/REC-xml-20081126/#NT-NameStartChar
 */
export declare function isNameStartChar(char: string, cp?: number): boolean;
/**
 * Returns `true` if _char_ is a valid reference character (which may appear
 * between `&` and `;` in a reference), `false` otherwise.
 *
 * @see https://www.w3.org/TR/2008/REC-xml-20081126/#sec-references
 */
export declare function isReferenceChar(char: string): boolean;
/**
 * Returns `true` if _char_ is an XML whitespace character, `false` otherwise.
 *
 * @see https://www.w3.org/TR/2008/REC-xml-20081126/#white
 */
export declare function isWhitespace(char: string): boolean;
/**
 * Returns `true` if _codepoint_ is a valid XML `Char` code point, `false`
 * otherwise.
 *
 * @see https://www.w3.org/TR/2008/REC-xml-20081126/#NT-Char
 */
export declare function isXmlCodePoint(cp: number): boolean;
//# sourceMappingURL=syntax.d.ts.map